/* umtsmon - a program to control/monitor your UMTS card
 * Copyright (C) 2007  Klaas van Gend
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation
 * applicable version is GPL version 2 only.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */


#include "umtsmon_global.h"
#include "AutoDetect.h"
#include "FileStuff.h"
#include "SerialPort.h"
#include "Query.h"
#include "Popup.h"
#include "Runner.h"

#include <assert.h>
#include <usb.h>

//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
Device*
AutoDetectBase::createDevice(void)
{
	DEBUG3(" AutoDetectBase::createDevice() for Device %x\n", theDeviceID);
	assert(theDeviceID);
	if (theDeviceID==0)
		return NULL;
		
	Device* myDevicePtr = Device::createDeviceInstance(theDeviceID);
	if (myDevicePtr != NULL)
	{
		assert(!thePPPSerialPortName.isNull());
		assert( !theATSerialPortName.isNull());
		
		if (thePPPSerialPortName.isEmpty())
			thePPPSerialPortName = theATSerialPortName;
		
		bool isPPPOK=  myDevicePtr->setupPPPPort(thePPPSerialPortName);
		bool isATOK =  myDevicePtr->setupATPort(theATSerialPortName);
		
		DEBUG4("PPP: %d, AT: %d\n", isPPPOK, isATOK);
		
		// both failed?
		if ((!isATOK) && (!isPPPOK))
			goto not_good;

		// either one failed or no fail - make it a single port thingie	if needed	
		if (isATOK && (!isPPPOK))
		{
			if (!myDevicePtr->setupPPPPort(theATSerialPortName))
				goto not_good;
			DEBUG5("  Will use port '%s' for PPP - only one port for AT+PPP\n", theATSerialPortName.ascii());
		}
		else
			DEBUG5("  Will use port '%s' for PPP\n", thePPPSerialPortName.ascii());
			
		if ((!isATOK) && isPPPOK)
		{
			if (!myDevicePtr->setupATPort(thePPPSerialPortName))
				goto not_good;
			DEBUG5("  Will use port '%s' for AT - only one port for AT+PPP\n",  thePPPSerialPortName.ascii());
		}
		else
			DEBUG5("  Will use port '%s' for AT\n",  theATSerialPortName.ascii());
	}
	
	sendLog(GOOD, "Device created successfully");
	return myDevicePtr;
	
not_good:
	sendLog(BAD, "Device couldn't be created");
	DEBUG1(" AutoDetectBase::createDevice() failed :-(\n"); 
	return NULL;	
}

//////////////////////////////////////////////////////////////////////////////
AutoDetectBase::AutoDetectResults 
AutoDetectBase::go(void)
{
	DEBUG3("AutoDetectBase::go() for '%s'\n", getName().ascii());
	AutoDetectResults myResult = traverseTrees();
	if (myResult == DEVICE_DETECTED)
	{
		if (!findPortNames())
			myResult = DEVICE_ERROR;
	} 
	return myResult;	
}
	

//////////////////////////////////////////////////////////////////////////////
void
AutoDetectBase::sendLog(LogQuality isGood, const QString& aMessage)
{
	// FIXME: for now, this is just a print to stdout.
	// in the future, this should be calling back a registered function that
	// either dumps to stdout, a log file or to a text view window
	QString myQuality;
	switch (isGood)
	{
	case GOOD:	
		myQuality="GOOD";
		DEBUG2("GOOD: '%s'\n", aMessage.ascii());
		break;
	case BAD:	
		myQuality="BAD";
		DEBUG2("BAD: '%s'\n", aMessage.ascii());
		break;
	case KILLING:	
		myQuality="KILLING";
		DEBUG1("KILLING: '%s'\n", aMessage.ascii());
		break;
	case INFO:	
		myQuality="INFO";
		DEBUG3("INFO: '%s'\n", aMessage.ascii());
		break;
	}
}



//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
AutoDetectBase::AutoDetectResults 
AutoDetect_USB::traverseTrees(void)
{
	DEBUG3(" AutoDetect_USB::traverseTrees()\n");
	usb_init();
    usb_find_busses();
    usb_find_devices();
	
	struct usb_bus* bus = NULL;
	
	for (bus = usb_get_busses(); bus; bus = bus->next) 
	{
		struct usb_device* dev = NULL;
		for (dev = bus->devices; dev; dev = dev->next) 
		{
			DEBUG4("  Checking USB device on %s:%s\n", bus->dirname, dev->filename);		
			switch(matchIDs(dev->descriptor.idVendor, dev->descriptor.idProduct))
			{
			case NOTHING_FOUND:
				break;
			case DEVICE_DETECTED:
				theUSBVendorID = dev->descriptor.idVendor;
				theUSBDeviceID = dev->descriptor.idProduct;
				return DEVICE_DETECTED;
			case DEVICE_ERROR:
				return DEVICE_ERROR;
			}
		}
	}
	// FIXME: if we got here, we couldn't find a device by ID
	// let's check if we can find any usbserial or usbmodems (ttyACM) this way

	sendLog(BAD, "No USB-based device found"); 
	return NOTHING_FOUND;
}


//////////////////////////////////////////////////////////////////////////////
AutoDetectBase::AutoDetectResults 
AutoDetect_USB::matchIDs(unsigned long aVendorID, unsigned long aProductID)
{
	DEBUG3(" AutoDetect_USB::matchIDs(0x%04lx, 0x%04lx)\n", aVendorID, aProductID);

struct s_args
{
	unsigned long theUSBVendorID;
	unsigned long theUSBProductID;
	QString       theGOODString;
	Device::CardType theDeviceID;
};


static struct s_args theArgsTable[] =
{ 
	//VENDOR, PRODUCT,     DESCRIPTION,           DEVICE TYPE
	//-------------------------------------------------------------------
	
	//////////////////// HUAWEI
	{ 0x12d1,  0x1001,     "Huawei E620",         Device::HUAWEI_MULTI, },
	{ 0x12d1,  0x1003,     "Huawei E220 / E800",  Device::HUAWEI_MULTI, },
	// catch-all Huawei, keep last:
	{ 0x12d1,  0,          "Unknown Huawei card", Device::HUAWEI_MULTI, },
	
	//////////////////// NOVATEL
	{ 0x1410, 0x1400,     "Novatel U740",        Device::NOVATEL_2PORT, },
	{ 0x1410, 0x1410,     "Novatel U740",        Device::NOVATEL_2PORT, },
	{ 0x1410, 0x1430,     "Novatel XU870 HSDPA", Device::NOVATEL_2PORT, },
	{ 0x8118, 0x143c,     "Dell Wireless 5510 (=Novatel XU870)", Device::NOVATEL_2PORT, },
	{ 0x413c, 0x8116,     "Dell/Novatel Wireless HSDPA",  Device::NOVATEL_2PORT, }, 
	// catch-all Novatel, keep last:
	{ 0x1410, 0,          "Unknown Novatel",     Device::NOVATEL_2PORT, },
	
	//////////////////// SIERRA WIRELESS (& alike)
	{ 0x05C6, 0x6613,     "ONDA N501HS / H600",  Device::OPTION_3PORT, },
	{ 0x1199, 0x6820,     "Sierra Wireless Aircard 875", Device::NOVATEL_2PORT, },
		
	
	//////////////////// OPTION
	{ 0x1955, 0x1000, "Option Icon or similar", Device::OPTION_3PORT, },
	{ 0x05c6, 0x1000, "Option Icon in USB Mass Storage Mode - please switch", Device::UNKNOWN, },
	// keep the specialised one (0x5000 for V3G) first, the catch-all (0 for generic Options) last
	{ 0x0af0, 0x5000,  "orig Vodafone 3G",      Device::OPTION_VODAFONE_3G, },
	// the following is info from: http://www.teltonika.lt/uploads/docs/3g_linux.pdf
	{ 0x0af0, 0x6501,  "Teltonika ModemUSB",    Device::OPTION_3PORT, },   
	{ 0x0af0, 0x6600,  "Option Icon as modem",  Device::OPTION_3PORT, },
	{ 0x0af0, 0x6901,  "Option Icon as modem",  Device::OPTION_3PORT, },
	{ 0x0af0,      0,  "Option GT family",      Device::OPTION_3PORT, },
	
	//////////////////// 4G SYSTEMS	
	{ 0x1955, 0x4e44, "4GSystems USB Box", Device::UNKNOWN, },
	{ 0x11f5,      0, "4GSystems USB Box",   Device::GENERIC_USER_SERIAL, },
	
	//////////////////// PANTECH
	// also known as Verizon PC5750, doesn't work with umtsmon
	{ 0x106c, 0x3702, "PANTECH/Verizon PC5750",   Device::UNKNOWN, },
	
	// keep this one last 
	{ 0, 0, "", Device::UNKNOWN },
};

	int i;
	for (i=0; theArgsTable[i].theUSBVendorID; i++)
	{
		if (aVendorID == theArgsTable[i].theUSBVendorID)
		{
			// also need to match product ID?
			if (theArgsTable[i].theUSBProductID != 0)
			{
				if (aProductID != theArgsTable[i].theUSBProductID)
				{
					continue;
				}
			}
			// if we get here, we're cool
			theDeviceID = theArgsTable[i].theDeviceID;
			sendLog(GOOD, "A '" + theArgsTable[i].theGOODString + "' modem detected");
			
			if (theDeviceID == Device::UNKNOWN)
			{
				sendLog(KILLING, "However, this device is not supported");
				return NOTHING_FOUND;
			}
			return DEVICE_DETECTED;
		}
	}

	return NOTHING_FOUND;	
}

//////////////////////////////////////////////////////////////////////////////
bool AutoDetect_USB::findPortNames(void)
{
	// Unfortunately, this info is not available from libusb :-(
	DEBUG3(" AutoDetect_USB::findPortNames()\n");

	QStringList myUSBPortList;
	int myUSBSerialPort;
	for (myUSBSerialPort=0; myUSBSerialPort<32; myUSBSerialPort++)
	{
		QString myttyUSBx = "ttyUSB" + QString::number(myUSBSerialPort);

		// we have to check for a lot of devices - due to insert/remove/insert/remove issues
		if (!FileStuff::doesFileExist("/sys/class/tty/" + myttyUSBx))
			continue;

		// a USB serial port was found!
		myUSBPortList.push_back(myttyUSBx);
		DEBUG4("  Found serial port number %d with name '%s'\n", myUSBPortList.count(), myttyUSBx.ascii());
	}

	if (myUSBPortList.empty())
	{
		// suggestModprobe() will already run this member again
//		return suggestModprobe();
		myUSBPortList.push_back("/dev/ttyACM0");
	}

	sendLog(INFO, "There are " + QString::number(myUSBPortList.count()) + " USB Serial ports available");

	// Many people reported that the last port of a Huawei device is not usable
	// so, let's remove it from the list...
	if (theUSBVendorID == 0x12D1) // HUAWEI
	{
		if (myUSBPortList.count() > 1)
			myUSBPortList.pop_back();
	}

	// note that last() may equal first() :-)
	theATSerialPortName = "/dev/" + myUSBPortList.last();
	thePPPSerialPortName = "/dev/" + myUSBPortList.first();
	DEBUG3("set AT  serial to '%s'\n", theATSerialPortName.ascii()); 
	DEBUG3("set PPP serial to '%s'\n", thePPPSerialPortName.ascii()); 

	// probing of this port will be done later...
	return true;
}



//////////////////////////////////////////////////////////////////////////////
bool AutoDetect_USB::suggestModprobe(void)
{
	// if we get here, we detected something looking like a usb2serial device
	// yet we didn't see any ttyUSB ports.
	// darn - let's see if the usbserial driver is loaded and suggest 
	// to load it otherwise
	DEBUG4("AutoDetect_USB::suggestModprobe(void)\n");
	if (FileStuff::doesFileContainString("/proc/modules", QString("usbserial")))
	{
		DEBUG1("no /dev/ttyUSB* entries, but the usbserial module is loaded???\n");
		return false;
	}
	
	QString myMessage = QObject::tr("Your device was recognised as a "
		"usbserial device,\nyet the corresponding kernel module is not "
		"running on your system.\n\nDo you want to load that module now?");
	if (Popup::YesNoQuestion(myMessage))
	{
		Runner myRunner;
		char myCommandString[256];
		snprintf(myCommandString, 255, "/sbin/modprobe usbserial vendor=0x%lx product=0x%lx", theUSBVendorID, theUSBDeviceID);
		DEBUG1("%s\n", myCommandString);
		myRunner.runCommand(Runner::SOMESU, myCommandString);
		
		// we now have to wait and see...
		sleep(1);
		if (findPortNames())
			return true;
	}
	return false;	
}		



//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
AutoDetectBase::AutoDetectResults 
AutoDetect_PCMCIA::traverseTrees(void)
{
	DEBUG3(" AutoDetect_PCMCIA::traverseTrees()\n");
	if (!isPCMCIABrowsingPossible()) 
		return NOTHING_FOUND;

	// work from thePCMCIASocketsDirectory here
	// it will contain at least one and maybe multiple pcmcia_socket dirs
    // each containing a link to a device
	QString mySockName;
	FileStuff::FindInDir myPCMCIASockDir = FileStuff::FindInDir("/sys/class/pcmcia_socket", 
				"pcmcia_socket*", QDir::Dirs);
	while( !( (mySockName=myPCMCIASockDir.getNextEntry()) .isEmpty()) )
	{
		QString myPCMCIASocketDevice = mySockName + "/device";
		QFileInfo mySockDev(myPCMCIASocketDevice);
		if (!mySockDev.isSymLink())
			continue;

		QString myPCMCIASocketDeviceLink = mySockDev.readLink();

		// the directory contains several entries, usually starting with 0000. 
		// So let's iterate over those entries and check for Vendor IDs
		FileStuff::FindInDir myPCMCIADir = FileStuff::FindInDir(
					myPCMCIASocketDevice, "0000:*", QDir::Dirs);
		QString my0000DirName;
		while( !( (my0000DirName=myPCMCIADir.getNextEntry()) .isEmpty()) )
		{
			QString myVendorID;
			QString myDeviceID;
			QString mySubVendorID;
			
			QString myTemp = my0000DirName + "/vendor";
			if (!FileStuff::doesFileExist(myTemp))
				sendLog(BAD, "Could not open file: " + myTemp);
			else
				myVendorID = FileStuff::getFileFirstLine(myTemp);
			
			myTemp = my0000DirName + "/device";
			if (!FileStuff::doesFileExist(myTemp))
				sendLog(BAD, "Could not open file: " + myTemp);
			else
				myDeviceID = FileStuff::getFileFirstLine(myTemp);
			
			switch (matchIDs(myVendorID.toULong(NULL,0), myDeviceID.toULong(NULL,0)))
			{
			case DEVICE_DETECTED:
				return DEVICE_DETECTED;
				break;
			case DEVICE_ERROR:
				// FIXME: This may be the wrong action here
				// as there might be another, good, device in this list
				return DEVICE_ERROR;
				break;
			case NOTHING_FOUND:
				DEBUG5("PCMCIA card found, but not detected: '%s'\n", my0000DirName.ascii());
				DEBUG5("VendorID:    0x%4s\n", myVendorID.ascii());
				DEBUG5("DeviceID:    0x%4s\n", myDeviceID.ascii());
				myTemp = my0000DirName + "/subsystem_vendor";
				if (FileStuff::doesFileExist(myTemp))
					mySubVendorID = FileStuff::getFileFirstLine(myTemp);
				else
					mySubVendorID = "<<not present>>";
				DEBUG5("SubVendorID: 0x%4s\n", mySubVendorID.ascii());
				break;
			} // end-of-switch
		} // end-of-while
	} // end-of-while
	sendLog(BAD, "no PCMCIA device found");
	return NOTHING_FOUND;
}


//////////////////////////////////////////////////////////////////////////////
AutoDetectBase::AutoDetectResults 
AutoDetect_PCMCIA::matchIDs(unsigned long aVendorID, unsigned long aProductID)
{
	DEBUG3(" AutoDetect_PCMCIA::matchIDs(0x%04lx, 0x%04lx)\n", aVendorID, aProductID);
	switch(aVendorID)
	{
	case 0x1931:
		sendLog(GOOD, "Detected Option Nozomi based card \"Nozomi\" PCMCIA card");
		theDeviceID = Device::OPTION_NOZOMI;
		return DEVICE_DETECTED;
		break;
	case 0x14e4:
	{
		sendLog(INFO, "Detected BroadCom-based PCMCIA card");
		sendLog(INFO, "Broadcom device ID: 0x" + QString::number(aProductID,16));
		if (aProductID == 0x4322)
		{
			sendLog(GOOD, "Detected BroadCom-based PCMCIA card with 4306 UART, probably Sony Ericsson GC79");
			theDeviceID = Device::BROADCOM_GPRS_EDGE;
			return DEVICE_DETECTED;
		}
		if (aProductID == 0x4333)
		{
			sendLog(GOOD, "Detected BroadCom-based PCMCIA card Option GT EDGE");
			theDeviceID = Device::BROADCOM_57K6;
			return DEVICE_DETECTED;
		}
		if (aProductID == 0x4344)
		{
			sendLog(GOOD, "Detected BroadCom-based Sony Ericsson GC86");
			theDeviceID = Device::BROADCOM_57K6;
			return DEVICE_DETECTED;
		}
		sendLog(BAD, "Unknown BroadCom device detected - might be just wifi card");	
		break;
	}
	case 0x1045: // OPTI
	case 0x1033: // NEC
		sendLog(KILLING, "Detected a USB-type PCMCIA card - should have been detected already");
		return DEVICE_ERROR;
		break;
	} // end-of-switch
	
	// nothing we know of :-(
	return NOTHING_FOUND;
} 


//////////////////////////////////////////////////////////////////////////////
bool AutoDetect_PCMCIA::findPortNames(void)
{
	DEBUG3(" AutoDetect_PCMCIA::findPortNames()\n");
	assert(theDeviceID);
	assert(theATSerialPortName.isEmpty());
	assert(thePPPSerialPortName.isEmpty());
	
	switch(theDeviceID)
	{
	case Device::BROADCOM_GPRS_EDGE:
	case Device::BROADCOM_57K6:
		return find4306PortNames();
		break;
	case Device::OPTION_NOZOMI:
		return findNozomiPortNames();
		break;
	default:
		// should not happen!!!
		assert(false);
	}	
	return false;
}

//////////////////////////////////////////////////////////////////////////////
bool AutoDetect_PCMCIA::findNozomiPortNames(void)
{
	DEBUG3(" AutoDetect_PCMCIA::findNozomiPortNames()\n");
	if (!FileStuff::doesFileContainString("/proc/modules", QString("nozomi")))
	{
		sendLog(KILLING, "No kernel module for nozomi-based card loaded");
		return false;
	}
	if (FileStuff::doesFileContainString("/proc/devices", QString("nozomi")))
	{
		theATSerialPortName  = "/dev/nozomi2";
		thePPPSerialPortName = "/dev/nozomi0";
		sendLog(GOOD, "found /dev/nozomi* - the 'Holtzman' version of Nozomi drivers");
	}
	else
	{
		if (FileStuff::doesFileContainString("/proc/devices", QString("noz")))
		{
			theATSerialPortName = "/dev/noz2";
			thePPPSerialPortName = "/dev/noz0";
			sendLog(GOOD, "found /dev/noz* - the 'Hardwick' version of Nozomi drivers.");
		}
		else
		{
			sendLog(BAD, "No kernel device nozomi found");
			return false;
		}
	}
	return true;
}
	
//////////////////////////////////////////////////////////////////////////////
bool AutoDetect_PCMCIA::find4306PortNames(void)
{
	DEBUG3(" AutoDetect_PCMCIA::find4306PortNames()\n");
	// apparently, (in higher kernels than 2.6.13-rc1???), there is a direct link to the serial port:
	// klaas@jip:/> find /sys/class/pcmcia_socket/pcmcia_socket0/device/ -name '*tty*'
	//   /sys/class/pcmcia_socket/pcmcia_socket0/device/0000:03:00.1/tty:ttyS2
	//   /sys/class/pcmcia_socket/pcmcia_socket0/device/0000:03:00.1/tty:ttyS1

	FileStuff::FindInDir myDevDir = FileStuff::FindInDir(
				"/sys/class/pcmcia_socket/pcmcia_socket0/device", 
				"0000:*", QDir::Dirs);
	QString my0000DirName;
	QString myFileName;
	while( !( (my0000DirName=myDevDir.getNextEntry()) .isEmpty()) )
	{
		FileStuff::FindInDir myDir = FileStuff::FindInDir(my0000DirName, 
					"tty:*", QDir::Dirs);
		while( !( (myFileName=myDir.getNextEntry()) .isEmpty()) )
		{
			QString theDevName = "/dev/" + myFileName.section(':', -1);
			theATSerialPortName = theDevName;
			thePPPSerialPortName = theDevName;
			sendLog(GOOD, "Detected Broadcom serial port on " + 
						theATSerialPortName);
			return true;
		}
	}
	return false;
}



//////////////////////////////////////////////////////////////////////////////
bool 
AutoDetect_PCMCIA::isPCMCIABrowsingPossible(void)
{ 
	// check for pcmcia_core in /proc/modules (=lsmod)
	if (FileStuff::doesFileContainString("/proc/modules", QString("pcmcia_core")))
	{
		sendLog(INFO, "pcmcia_core kernel support (yenta) found");
	}
	else
	{
		sendLog(BAD, "pcmcia_core kernel support (yenta) not found");
	}
	if (FileStuff::doesFileContainString("/proc/devices", QString("pcmcia")))
	{
		sendLog(INFO, "PCMCIA kernel module found - cardctl support possible");
	}
	else
	{	
		sendLog(BAD, "PCMCIA kernel module not found - enabling/disabling pccards not possible");
	}

	// check for pcmcia in /sys/class
	if (FileStuff::doesFileExist("/sys/class/pcmcia_socket"))
	{
		sendLog(INFO, "pcmcia browsing available");
		return true;
	}
	else
	{
		sendLog(BAD, "PCMCIA browsing in SysFS not available - no autodetection");
		return false;
	}
}


//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
AutoDetectBase::AutoDetectResults 
AutoDetect_Rawport::traverseTrees(void)
{
	DEBUG3(" AutoDetect_Rawport::traverseTrees()\n");
	assert(!theDevName.isEmpty());
	// let's  iterate through all known ports defined above
	// and keep a list of everything that at least can be opened
	theValidSerialPortsList.clear();
	QString myTTYName;

	DEBUG3("  Checking for ports of type /dev/%s\n", theDevName.ascii());
	FileStuff::FindInDir myDevDir = FileStuff::FindInDir("/dev", 
				theDevName, 
				static_cast<QDir::FilterSpec>(QDir::All | QDir::System | QDir::Hidden));
	while( !( (myTTYName=myDevDir.getNextEntry()) .isEmpty()) )
	{
		DEBUG3("   Checking properties of port %s\n", myTTYName.ascii());
		SerialPort mySerial;
		if (mySerial.openDev(myTTYName))
		{
			QString myMessage = "Serial port " + myTTYName + " is operational"; 
			sendLog(GOOD, myMessage);
			theValidSerialPortsList.push_back(myTTYName);
		}
		else
		{
			DEBUG3("     - Nothing on port %s\n", myTTYName.ascii()); 
		}
		mySerial.closeDev();
	}
	
	DEBUG3("end of traverseTrees()\n");
	if (theValidSerialPortsList.isEmpty())
	{
		sendLog(BAD, "No usable device found");
		return NOTHING_FOUND;
	}
	else
	{
		// this is a little hopeful... we only know that
		// such serial ports exists for now...
		DEBUG3("There are %d devices in my valid list\n",theValidSerialPortsList.count()); 
		return DEVICE_DETECTED;
	}
}


//////////////////////////////////////////////////////////////////////////////
AutoDetectBase::AutoDetectResults 
AutoDetect_Rawport::matchIDs(__attribute__ ((unused)) unsigned long aVendorID, 
                             __attribute__ ((unused)) unsigned long aProductID)
{

	// nothing we know - serial ports don't have vendor or product ids :-(
	return NOTHING_FOUND;
} 


// this class is used in AutoDetect_Rawport::findPortNames(void)
class s_my_pair
{
public:
	s_my_pair(void) {DEBUG1("S_MY_PAIR VOID\n");};
	s_my_pair(const QString& pn, int bc) : thePortName(pn), theBaudCode(bc) {DEBUG1("S_MY_PAIR PARAM\n");};
	QString thePortName;
	int     theBaudCode;
};


//////////////////////////////////////////////////////////////////////////////
bool AutoDetect_Rawport::findPortNames(void)
{
	int aBaudCode;
	DEBUG3(" AutoDetect_Rawport::findPortNames()\n");
	
struct s_args
{
	int           aBaudCode;
	unsigned long aBaudRate;
};

// list of baud rates to try
static struct s_args myBaudRates[] =
{
//	{B4000000, 4000000},
//	{B1152000, 1152000},
//	{B576000,   576000},
//	{B460800,   460800},
//	{B230400,   230400},
	{B115200,   115200},
	{B57600,     57600},
	{B38400,     38400},
	{B19200,     19200},
	{B9600,       9600},
	{0, 0}, 
};


//	list<s_my_pair>  myPortList;
	QValueList<s_my_pair>  myPortList;

	// we now have a list of valid serial devices in theValidSerialPortsList
	// we need to iterate through that list to see if there's a response to AT
	QStringList::Iterator myIt;
	for ( myIt = theValidSerialPortsList.begin(); myIt != theValidSerialPortsList.end(); ++myIt ) 
	{
		DEBUG5(" - probing for AT on device: %s\n", (*myIt).ascii());
		int i;
		for (i=0; myBaudRates[i].aBaudCode != 0; i++)
		{
			SerialPort mySerial;
			DEBUG5("    - testing baud rate %ld bps\n", myBaudRates[i].aBaudRate);
			if (mySerial.openDev(*myIt, myBaudRates[i].aBaudCode) == false)
			   continue;
			Query myQueryAT("AT");
			Query::ReturnCode myResult = myQueryAT.run();
			// unknown error probably means "i did get an answer, but no echo"...
	 		if (myResult == Query::OK) // || myResult == Query::UNKNOWN_ERROR)
	 		{
	 			DEBUG3("probe successful for baud rate %ld\n", myBaudRates[i].aBaudRate);
	 			myPortList.push_back(s_my_pair(*myIt,myBaudRates[i].aBaudRate));
	 			aBaudCode = myBaudRates[i].aBaudCode;
	 			break;
	 		}
		}
	}
	if (myPortList.empty()==false)
	{
		theATSerialPortName = myPortList.last().thePortName;
		thePPPSerialPortName= myPortList.first().thePortName;
		theDeviceID = DeviceCapabilities::GENERIC_USER_SERIAL;
		QString myMessage = "Found Serial ports AT:" + theATSerialPortName + ", PPP:" + thePPPSerialPortName; 
		sendLog(GOOD, myMessage);
	}

	DEBUG3(" end of AutoDetect_Rawport::findPortNames()\n");
	return (myPortList.empty()==false);
}




//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
AutoDetectBase::AutoDetectResults 
AutoDetect_PhoneModems::go(void)
{
	DEBUG3("AutoDetect_PhoneModems::go()\n");
	theDevName = "ttyACM*";
	AutoDetectResults myResult = AutoDetectBase::go();	
	if (myResult == DEVICE_DETECTED)
	{
		theDeviceID = DeviceCapabilities::MODEM_ACM;
	}
	else
	{
		theDevName = "rfcomm*";
		myResult = AutoDetectBase::go();	
		if (myResult == DEVICE_DETECTED)
		{
			theDeviceID = DeviceCapabilities::MODEM_RFCOMM;
		}
	} 
	return myResult;	
}


//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
AutoDetectBase::AutoDetectResults 
NonAutoDetect::matchIDs(__attribute__ ((unused)) unsigned long aVendorID, 
                             __attribute__ ((unused)) unsigned long aProductID)
{
	// nothing we know - serial ports don't have vendor or product ids :-(
	return NOTHING_FOUND;
} 

//////////////////////////////////////////////////////////////////////////////
AutoDetectBase::AutoDetectResults 
NonAutoDetect::go(void)
{
	if (theForceAutodetection)
	{
		DEBUG2("Force Autodetection enabled - skipping device instantiation from supplied values\n");
		return NOTHING_FOUND;
	}


	if (theSuggestedDeviceType == DeviceCapabilities::UNKNOWN)
	{
		// nothing in config
		DEBUG2("No valid configuration available in config file\n"); 
		return NOTHING_FOUND;
	}
	else
	{
		// let's try with the supplied parameters
		theDeviceID = theSuggestedDeviceType;
		theATSerialPortName = theSuggestedATPortName;
		thePPPSerialPortName = theSuggestedPPPPortName;
		return AutoDetectBase::go();
	}
} 

